@@ -16,6 +16,9 @@ You should have received a copy of the GNU General Public License along with
 kppy.  If not, see <http://www.gnu.org/licenses/>.
 '''
 
+import binascii
+import sys
+import os
 import struct
 from datetime import datetime
 from os import remove, path
@@ -824,27 +827,31 @@ class KPDBv1(object):
     def _get_filekey(self):
         """This method creates a key from a keyfile."""
 
-        with open(self.keyfile, 'rb') as handler:
-            try:
-                buf = handler.read()
-            except:
-                raise KPError('Could not read file.')
-        sha = SHA256.new()
-        if len(buf) == 33:
-            sha.update(buf)
-            return sha.digest()
-        elif len(buf) == 65:
-            sha.update(struct.unpack('<65s', buf)[0].decode())
-            return sha.digest()
-        else:
-            while buf:
-                if len(buf) <= 2049:
+        if not os.path.exists(self.keyfile):
+                raise KPError('Keyfile not exists.')
+        try:
+            with open(self.keyfile, 'rb') as handler:
+                handler.seek(0,os.SEEK_END)
+                size = handler.tell()
+                handler.seek(0,os.SEEK_SET)
+
+                if size == 32:
+                    return handler.read(32)
+                elif size == 64:
+                    try:
+                        return binascii.unhexlify(handler.read(64))
+                    except (TypeError,binascii.Error):
+                        handler.seek(0,os.SEEK_SET)
+
+                sha = SHA256.new()
+                while True:
+                    buf = handler.read(2048)
                     sha.update(buf)
-                    buf = []
-                else:
-                    sha.update(buf[:2048])
-                    buf = buf[2048:]
-            return sha.digest()
+                    if len(buf) < 2048:
+                        break
+                return sha.digest()
+        except IOError as e:
+            raise KPError('Could not read file: %s' % e)
 
     def _cbc_decrypt(self, final_key, crypted_content):
         """This method decrypts the database"""
@@ -853,6 +860,10 @@ class KPDBv1(object):
         aes = AES.new(final_key, AES.MODE_CBC, self._enc_iv)
         decrypted_content = aes.decrypt(crypted_content)
         padding = decrypted_content[-1]
+        if sys.version > '3':
+            padding = decrypted_content[-1]
+        else:
+            padding = ord(decrypted_content[-1])
         decrypted_content = decrypted_content[:len(decrypted_content)-padding]
         
         return decrypted_content
@@ -880,13 +891,11 @@ class KPDBv1(object):
             group.id_ = struct.unpack('<I', decrypted_content[:4])[0]
         elif field_type == 0x0002:
             try:
-                group.title = str(struct.unpack('<{0}s'.format(field_size-1),
-                                            decrypted_content[:field_size-1])[0],
-                                            'utf-8')
+                group.title = struct.unpack('<{0}s'.format(field_size-1),
+                    decrypted_content[:field_size-1])[0].decode('utf-8')
             except UnicodeDecodeError:
-                group.title = str(struct.unpack('<{0}s'.format(field_size-1),
-                                            decrypted_content[:field_size-1])[0],
-                                            'latin-1')
+                group.title = struct.unpack('<{0}s'.format(field_size-1),
+                    decrypted_content[:field_size-1])[0].decode('latin-1')
             decrypted_content = decrypted_content[1:]
         elif field_type == 0x0003:
             group.creation = self._get_date(decrypted_content)
@@ -924,28 +933,23 @@ class KPDBv1(object):
         elif field_type == 0x0003:
             entry.image = struct.unpack('<I', decrypted_content[:4])[0]
         elif field_type == 0x0004:
-            entry.title = str(struct.unpack('<{0}s'.format(field_size-1),
-                                        decrypted_content[:field_size-1])[0],
-                                        'utf-8')
+            entry.title = struct.unpack('<{0}s'.format(field_size-1),
+                    decrypted_content[:field_size-1])[0].decode('utf-8')
             decrypted_content = decrypted_content[1:]
         elif field_type == 0x0005:
-            entry.url = str(struct.unpack('<{0}s'.format(field_size-1),
-                                        decrypted_content[:field_size-1])[0],
-                                        'utf-8')
+            entry.url = struct.unpack('<{0}s'.format(field_size-1),
+                    decrypted_content[:field_size-1])[0].decode('utf-8')
             decrypted_content = decrypted_content[1:]
         elif field_type == 0x0006:
-            entry.username = str(struct.unpack('<{0}s'.format(field_size-1),
-                                        decrypted_content[:field_size-1])[0],
-                                         'utf-8')
+            entry.username = struct.unpack('<{0}s'.format(field_size-1),
+                    decrypted_content[:field_size-1])[0].decode('utf-8')
             decrypted_content = decrypted_content[1:]
         elif field_type == 0x0007:
-            entry.password = str(struct.unpack('<{0}s'.format(field_size-1),
-                                        decrypted_content[:field_size-1])[0],
-                                        'utf-8')
+            entry.password = struct.unpack('<{0}s'.format(field_size-1),
+                    decrypted_content[:field_size-1])[0].decode('utf-8')
         elif field_type == 0x0008:
-            entry.comment = str(struct.unpack('<{0}s'.format(field_size-1),
-                                        decrypted_content[:field_size-1])[0],
-                                        'utf-8')
+            entry.comment = struct.unpack('<{0}s'.format(field_size-1),
+                    decrypted_content[:field_size-1])[0].decode('utf-8')
         elif field_type == 0x0009:
             entry.creation = self._get_date(decrypted_content)
         elif field_type == 0x000A:
@@ -955,9 +959,8 @@ class KPDBv1(object):
         elif field_type == 0x000C:
             entry.expire = self._get_date(decrypted_content)
         elif field_type == 0x000D:
-            entry.binary_desc = str(struct.unpack('<{0}s'.format(field_size-1),
-                                       decrypted_content[:field_size-1])[0],
-                                       'utf-8')
+            entry.binary_desc = struct.unpack('<{0}s'.format(field_size-1),
+                    decrypted_content[:field_size-1])[0].decode('utf-8')
         elif field_type == 0x000E:
             entry.binary = decrypted_content[:field_size]
         elif field_type == 0xFFFF:
